--[[
	Name: GridTargets
	Revision: $Revision: 82146 $
	Developed by: Julith @ Perenolde-EU (billgatesgut@web.de)
]]

--{{{ Libraries

local L = AceLibrary("AceLocale-2.2"):new("GridTargets")

--}}}

L:RegisterTranslations("enUS", function()
	return {
		    ["GridTargets"] = true,
            ["mob health"] = true,
            ["mob target"] = true,
            ["mob cast"] = true,
            ["mob aggro"] = true,
            ["Use class color"] = true,
            ["Color target based on class."] = true,
            ["Ignore self"] = true,
            ["Does not display if you are the target"] = true,
            ["Use threat color"] = true,
            ["Color health based on your threat on the mob."] = true,
            ["Tanking"] = true,
            ["Color for Tanking."] = true,
            ["Almost lost"] = true,
            ["Color for still tanking but almost lost."] = true,
            ["Lost aggro"] = true,
            ["Color for not tanking."] = true,
            ["Color by mob ID"] = true,
            ["Color health based on the mobs id. This tries to give every mob a distinct color."] = true,
	}
end)

GridTargets = GridStatus:NewModule("GridTargets")
GridTargets.menuName = L["GridTargets"]


--{{{ AceDB defaults

local BY_COLOR = 1
local BY_THREAT = 2
local BY_ID = 3

GridTargets.defaultDB = {
	mobHealth = {
		enable = true,
		priority = 90,
		color = { r = 0.5, g = 0.5, b = 0.5, a = 1 },
        threatColor = { 
            [3] = { r = 0, g = 1, b = 0, a = 1 },
            [2] = { r = 0.5, g = 0.5, b = 0, a = 1 },
            [1] = { r = 1, g = 0, b = 0, a = 1 },
        },
        useThreatColor = true,
        colorMode = BY_THREAT,
        ignoreSelf = true,
	},
    mobTarget = {
		enable = true,
		priority = 90,
		color = { r = 1, g = 0, b = 0, a = 1 },
        useClassColor = true,
        ignoreSelf = true,
	},
    mobCast = {
        enable = true,
        priority = 90,
        color = { r = 1, g = 0, b = 0, a = 1 },
    },
    updateInterval = 0.1
}

--}}}

local function getstatuscolor(status)
    if status == 0 then status = 1 end
    local color = GridTargets.db.profile.mobHealth.threatColor[status]
    return color.r, color.g, color.b, color.a
end

local function setstatuscolor(status, r, g, b, a)
    local color = GridTargets.db.profile.mobHealth.threatColor[status]
    color.r = r
    color.g = g
    color.b = b
    color.a = a or 1
end

--{{{ Options
local options = {
    [1] = {
        type = "color",
        name = L["Tanking"],
        desc = L["Color for Tanking."],
        order = 87,
        hasAlpha = true,
        get = function () return getstatuscolor(3) end,
        set = function (r, g, b, a) setstatuscolor(3, r, g, b, a) end,
    },
    [2] = {
        type = "color",
        name = L["Almost lost"],
        desc = L["Color for still tanking but almost lost."],
        order = 88,
        hasAlpha = true,
        get = function () return getstatuscolor(2) end,
        set = function (r, g, b, a) setstatuscolor(2, r, g, b, a) end,
    },
    [3] = {
        type = "color",
        name = L["Lost aggro"],
        desc = L["Color for not tanking."],
        order = 89,
        hasAlpha = true,
        get = function () return getstatuscolor(1) end,
        set = function (r, g, b, a) setstatuscolor(1, r, g, b, a) end,
    },
    ["useThreatColor"] = {
		type = "toggle",
		name = L["Use threat color"],
		desc = L["Color health based on your threat on the mob."],
		get = function ()
			      return GridTargets.db.profile.mobHealth.colorMode == BY_THREAT
		      end,
		set = function (v)
			      GridTargets.db.profile.mobHealth.colorMode = v and BY_THREAT or BY_COLOR
			      GridTargets:GMUpdate()
		      end,
	},
    ["colorById"] = {
		type = "toggle",
		name = L["Color by mob ID"],
		desc = L["Color health based on the mobs id. This tries to give every mob a distinct color."],
		get = function ()
			      return GridTargets.db.profile.mobHealth.colorMode == BY_ID
		      end,
		set = function (v)
			      GridTargets.db.profile.mobHealth.colorMode = v and BY_ID or BY_COLOR
			      GridTargets:GMUpdate()
		      end,
	},
    ["ignoreSelf"] = {
		type = "toggle",
		name = L["Ignore self"],
		desc = L["Does not display if you are the target"],
		get = function ()
			      return GridTargets.db.profile.mobHealth.ignoreSelf
		      end,
		set = function (v)
			      GridTargets.db.profile.mobHealth.ignoreSelf = v
			      GridTargets:GMUpdate()
		      end,
	},
}

local targetOptions = {
    ["useClassColors"] = {
		type = "toggle",
		name = L["Use class color"],
		desc = L["Color target based on class."],
		get = function ()
			      return GridTargets.db.profile.mobTarget.useClassColor
		      end,
		set = function (v)
			      GridTargets.db.profile.mobTarget.useClassColor = v
			      GridTargets:GMUpdate()
		      end,
	},
    ["ignoreSelf"] = {
		type = "toggle",
		name = L["Ignore self"],
		desc = L["Does not display if you are the target"],
		get = function ()
			      return GridTargets.db.profile.mobTarget.ignoreSelf
		      end,
		set = function (v)
			      GridTargets.db.profile.mobTarget.ignoreSelf = v
			      GridTargets:GMUpdate()
		      end,
	},
}

local castOptions = {
}
--}}}

local settings

function GridTargets:OnInitialize()
	self.super.OnInitialize(self)
	self:RegisterStatus("mobHealth", L["mob health"], options, true)
    self:RegisterStatus("mobTarget", L["mob target"], targetOptions, true)
    self:RegisterStatus("mobCast", L["mob cast"], castOptions, true)
    settings = GridTargets.db.profile
end

function GridTargets:OnEnable()
    self:RegisterEvent("UNIT_TARGET", "Update")
    self:RegisterEvent("Grid_UnitJoined")
	self:RegisterEvent("Grid_UnitChanged")
    
    self:RegisterEvent("UNIT_SPELLCAST_START", "UpdateTarget")
    self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START", "UpdateTarget")
    self:RegisterEvent("UNIT_SPELLCAST_STOP", "UpdateTarget")
    self:RegisterEvent("UNIT_SPELLCAST_FAILED", "UpdateTarget")
    self:RegisterEvent("UNIT_SPELLCAST_DELAYED", "UpdateTarget")
    self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE", "UpdateTarget")
    self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP", "UpdateTarget")
    self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED", "UpdateTarget")
    
    self:ScheduleRepeatingEvent("GMUpdate", "GMUpdate", settings.updateInterval)    
    self:RegisterEvent("GMUpdate")
end

function GridTargets:Grid_UnitJoined(unitGUID, unitid)
	self:Update(unitid)
end

function GridTargets:Grid_UnitChanged(unitGUID, unitid)
    self:Update(unitid)
end

function isMob(unitid)
    return UnitExists(unitid) and UnitIsEnemy("player", unitid) and not UnitIsDead(unitid)
end

local function GUIDToID(guid)
    return tonumber(guid:sub(-12,-7),16), tonumber(guid:sub(-6,-1),16)
end

local COLORS = {
    [1] = {r=1,g=1,b=0,a=1}, --yellow star
    [2] = {r=1,g=0.5,b=0,a=1}, --orange circle
    [3] = {r=1,g=0,b=1,a=1}, --purple diamond
    [4] = {r=0,g=1,b=0,a=1}, --green triangle
    [5] = {r=0.8,g=0.6,b=0.6,a=1}, --white moon
    [6] = {r=0,g=0,b=1,a=1}, --blue square
    [7] = {r=1,g=0,b=0,a=1}, -- red cross
    [8] = {r=1,g=1,b=1,a=1}, --white skull
    
    [9] = {r=0,g=1,b=1,a=1},
    [10] = {r=0,g=0.5,b=1,a=1},
    [11] = {r=1,g=0,b=0.5,a=1},
    [12] = {r=0,g=1,b=0.5,a=1},
    [13] = {r=0.5,g=0.5,b=1,a=1},
    [14] = {r=1,g=0.5,b=0.5,a=1},
    [15] = {r=0.5,g=1,b=0.5,a=1},
    [16] = {r=1,g=0.25,b=0,a=1},
    [17] = {r=0.25,g=1,b=0,a=1},
    [18] = {r=0,g=1,b=0.25,a=1},
    [19] = {r=0.25,g=0,b=1,a=1},
}
    

local MOB_COLORS = setmetatable({}, {
    __index = function(t, i)
        local mobID, spawnID = GUIDToID(i)
        mobID = mobID % 11 + 9
        --print(mobID)
        spawnID = spawnID % 4 - 1
        local c = {}
        c.r = COLORS[mobID].r * 0.7^spawnID
        c.g = COLORS[mobID].g * 0.7^spawnID
        c.b = COLORS[mobID].b * 0.7^spawnID
        c.a = 1.0
		rawset(t, i, c)
		return c
	end,
    }) 


function GridTargets:Update(unitid)
    local guid = UnitGUID(unitid)
    local target = unitid.."target"
    
    if not isMob(target) then
        self.core:SendStatusLost(guid, "mobHealth")
        self.core:SendStatusLost(guid, "mobTarget")
        self.core:SendStatusLost(guid, "mobCast")
        return
    end

    local health = UnitHealth(target)
    local maxHealth = UnitHealthMax(target)
    
    
    local status = UnitThreatSituation("player", target)
    
    if not status or (settings.mobHealth.ignoreSelf and status == 3) then
        self.core:SendStatusLost(guid, "mobHealth")
    else
        local tmpColor = {}
        if settings.mobHealth.colorMode == BY_THREAT and status then
            tmpColor.r, tmpColor.g, tmpColor.b, tmpColor.a = getstatuscolor(status)
        elseif settings.mobHealth.colorMode == BY_ID then
            if GetRaidTargetIndex(target) then
                tmpColor = COLORS[GetRaidTargetIndex(target)]
            else
                local mobGUID = UnitGUID(target)
                tmpColor = MOB_COLORS[mobGUID]
            end
        else
            tmpColor = settings.mobHealth.color
        end

        self.core:SendStatusGained(guid, "mobHealth",
                        settings.mobHealth.priority,
                        nil,
                        tmpColor,
                        nil,
                        health, maxHealth)
    end
                    
    local targetsTarget = target.."target"
    if not UnitExists(targetsTarget) or settings.mobTarget.ignoreSelf and UnitIsUnit("player", targetsTarget) then
        self.core:SendStatusLost(guid, "mobTarget")
        return
    end
    
    local name = UnitName(targetsTarget)
    local ttGuid = UnitGUID(targetsTarget)
    
    self.core:SendStatusGained(guid, "mobTarget",
				    settings.mobTarget.priority,
				    nil,
				    (settings.mobTarget.useClassColor and 
                     self.core:UnitColor(ttGuid) or
                     settings.mobTarget.color),
                    name)
                    
    
    local spell, rank, displayName, icon, startTime, endTime, isTradeSkill, castID, interrupt = UnitCastingInfo(target)
    
    if not spell then
        spell, rank, displayName, icon, endTime, startTime, isTradeSkill, castID, interrupt = UnitChannelInfo(target)
    end
    
    if spell then
        local tmpColor = {}
        if GetRaidTargetIndex(target) then
            tmpColor = COLORS[GetRaidTargetIndex(target)]
        else
            local mobGUID = UnitGUID(target)
            tmpColor = MOB_COLORS[mobGUID]
        end
        --print(("%s: %d of %d"):format(spell,abs(GetTime()*1000 - startTime),abs(endTime - startTime)))
        self.core:SendStatusGained(guid, "mobCast",
                        settings.mobCast.priority,
                        nil,
                        tmpColor,
                        spell,
                        abs(GetTime()*1000 - startTime), abs(endTime - startTime))
    else
        self.core:SendStatusLost(guid, "mobCast")
    end

end

function GridTargets:GMUpdate()
    for guid, unitid in GridRoster:IterateRoster() do
		self:Update(unitid)
	end
end

function GridTargets:UNIT_TARGET(unitid)    
    self:Update(unitid)
end

function GridTargets:UpdateTarget(unitid)  
    if not isMob(unitid) then return end
    
    --print("id: "..unitid)
    --print("sub: "..unitid:sub(-6))
    
    if unitid:sub(-6) ~= "target" then return end
    --print("len: "..unitid:len())
    local playerid
    if unitid:len() == 6 then
        playerid = "player"
    else
        playerid = unitid:sub(1, -7)
        --print("playerid: "..playerid)
        local guid = UnitGUID(playerid)
        if not GridRoster:IsGUIDInRaid(guid) then return end
    end
    
    self:Update(playerid)
end
